#include <stdio.h>
#include <math.h>
#include <string.h>


FILE* infile;
FILE* rfile;
FILE* outfile;

char ChunkID[5],Format[5],Subchunk1ID[5],Subchunk2ID[5];

int ChunkSize,Subchunk1Size,AudioFormat;
int NumChannels,SampleRate,ByteRate,BlockAlign;
int BitsPerSample,Subchunk2Size,BytesPerSample;
int pairs_per_sec;

int ifl, samples_per_block, block_size;

int ok, offset, i, io, irc;

double total_time, dt, block_duration, tstart, tnow;
double rate[1024];

char appdirname[1024];
char leafname[1024];
char outdirname[1024];
char outfilename[1024];

char infilename[1024];

int infileok, ok;

int tellme;

unsigned char datablock[5760064];
double left[960001], right[960001];
double rec_left[1000], rec_right[1000];

double left_rate[1000], right_rate[1000];

double fsnow[1000], left_a[1000], left_q[1000], right_a[1000], right_q[1000];

int hist_raw[501], hist_wet[501];


double dc_left,dc_right,rms_left,rms_right;
double f0_counter, rotang, drotang, re_bit, qu_bit ;
double raw_rms, aes_rms, raw_2sig, aes_2sig;


double read_header(void);
void get4(char*);
int get4int(void);
int get2int(void);

void load_data_block24(void);
void load_data_block16(void);

double find_counter_frequency(void);
double get_fi(double,double);
double interpolate(double,double);
void highpass(void);

int get_start_time(void);

void do_transform(void);
void regenerate(void);
double get_weight(double);
void histograms(void);

int main(int argc,char* argv[])
{
  
  
  printf("  LP_WowAndFlutter Ver 0.26 5th Nov 2019\n");
  printf("  =========================================\n\n");
  printf("This lets you carry out Wow and Flutter analysis.\n");
  printf("Please read the Help inside the application before first use.\n");
  printf("This version outputs Veusz files (Polar).\n");
  
  sprintf(appdirname,"%s",argv[0]);
  ok=(long)strrchr(appdirname,'/')-(long)appdirname;
  appdirname[ok]=(char)0;
  printf("Application directory = %s\n",appdirname);
  sprintf(outdirname,"%s/%s",appdirname,"Results");
  printf("Output into directory = %s\n",outdirname);
  
  if (argc>1)
  {
    infileok=1;
    sprintf(infilename,"%s",argv[1]);
    printf("Input from file = %s\n\n",infilename);
  }
  else
  {
    infileok=0;
    printf("Drag a WAV file and drop it on the application icon\n");
    printf("if you wish to examine the contents of the file.\n\n");
  }
  
  if(infileok==1)
  {
  
    ok=(long)strrchr(infilename,'/');
    sprintf(leafname,"%s",(char*)(ok+1));
    printf("Input file leafname = %s\n",leafname);
  
    total_time=read_header();
    dt=1.0/(double)pairs_per_sec;                 /* sampling interval */
    samples_per_block=10*SampleRate;              /* samples per channel for 10 sec */
    block_duration=dt*(double)samples_per_block;  /* Duration of block in seconds */
    block_size=2*samples_per_block*BytesPerSample;
    printf("sample interval = %9.3lf microsec\n",dt*1.0E6);
    printf("block size = %d  Duration = %6.2lf sec\n",samples_per_block,block_duration);
    
    
    offset=get_start_time();
    
    if(offset>0)
    {
      printf("reading from %s\n",infilename);
      infile=fopen(infilename,"rd");
      fseek(infile,(offset+44),SEEK_SET);
      
      if (BytesPerSample==3)
      {
        printf("Loading 24bit payload\n");
        load_data_block24();
      }
      else
      {
        printf("Loading 16bit payload\n");
        load_data_block16();
      }
      
      fclose(infile);
      
      printf("Left  dc = %6.4lf  rms = %6.4lf\n",dc_left,rms_left);
      printf("Right dc = %6.4lf  rms = %6.4lf\n",dc_right,rms_right);
      
      printf("Apply 200Hz highpass filter.\n");
      highpass();
      
      f0_counter=find_counter_frequency();
      
      printf("Found f0_counter = %6.4lf\n",f0_counter);
      
      printf("\nNow examine chunks\n");
      
      tnow=0.5; i=0;
      do
      {
        get_fi(tnow,0.01);
        i++;
        tnow+=0.01;
      } while (tnow<9.51);
      
      printf("Ended with i = %d\n",i);
      
      do_transform();  /* Obtain spectrum of the frequency variations */
      
      regenerate(); /* Now create the time-frequency behaviour with AES weighting */
      
      sprintf(outfilename,"%s/Polar.csv",outdirname);
      
      rfile=fopen(outfilename,"wt");

      fprintf(rfile,"\"rangle\",\"rawang\",\"wawang\"\n");
      
      drotang=0.01*360.0*0.5555555555;  /* Polar rate assumed to be 33.3333 rpm) */
      rotang=0.0;

      raw_rms=0.0; aes_rms=0.0;

      io=1;
      do
      {
        fprintf(rfile,"%6.4lf, ",rotang);
        re_bit= 0.5*(left_rate[io]+right_rate[io]); 
        qu_bit= 0.5*(rec_left[io]+rec_right[io]);
        fprintf(rfile,"%6.4lf, %6.4lf\n",re_bit,qu_bit);
        raw_rms += re_bit*re_bit; aes_rms += qu_bit*qu_bit;
        rotang+=drotang;
        io++;
      } while (rotang < 3.0*360.0);
      
      fclose(rfile);
      raw_rms=sqrt(raw_rms/(double)(io-1));  aes_rms=sqrt(aes_rms/(double)(io-1));    
      histograms();  /* Generate histograms of the time-frequency variations */       
    }
  }


  sprintf(outfilename,"%s/info.txt",outdirname);
  outfile=fopen(outfilename,"w");

  fprintf(outfile,"\"InputFile: %s \" ",leafname);
  fprintf(outfile,"\"Raw  %5.2lf %%  %5.2lf %%\" ",raw_rms,raw_2sig);
  fprintf(outfile,"\"AES  %5.2lf %%  %5.2lf %%\" ",aes_rms,aes_2sig);
  fprintf(outfile,"\"Ave. Frequency %6.1lf Hz\" ",f0_counter);
  fclose(outfile);

  printf("\nDone!\n");
  return 0;
}



void histograms(void)
{
  int ii, ioff, iw, iok, icount, rcount, isum_raw, isum_wet, ilimit, rlimit;
  int routcount; 
  double pcl, pcr, hrange, hscale;
  
  hrange=0.25; hscale=500.0; routcount=0;
  ii=100; icount=0; rcount=0;
  do
  {
    pcl=left_rate[ii]; pcr=right_rate[ii];
    ioff = 250 + (int) (hscale*pcl);
    iok=1;
    if(ioff<0)
    {
      iok=0; routcount++;
    }
    if(ioff>500)
    {
      iok=0; routcount++;
    }
    
    if (iok==1)
    {
      hist_raw[ioff] += 1;
      icount++;
    }
    
    ioff = 250 + (int) (hscale*pcr);
    iok=1;
    
    if(ioff<0)
    {
      iok=0; routcount++;
    }
    if(ioff>500)
    {
      iok=0; routcount++;
    }
    
    if (iok==1)
    {
      hist_raw[ioff] += 1;
      icount++;
    }
    
    pcl=rec_left[ii]; pcr=rec_right[ii];
    ioff = 250 + (int) (hscale*pcl);
    iok=1;
    
    if(ioff<0)
    {
      iok=0; routcount++;
    }
    if(ioff>500)
    {
      iok=0; routcount++;
    }
    
    if (iok==1)
    {
      hist_wet[ioff] += 1;
      rcount++;
    }
    
    ioff = 250 + (int) (hscale*pcr);
    iok=1;
    
    if(ioff<0)
    {
      iok=0; routcount++;
    }
    if(ioff>500)
    {
      iok=0; routcount++;
    }
    
    if (iok==1)
    {
      hist_wet[ioff] += 1;
      rcount++;
    }
     
    ii++;
  } while (ii<900);

  if(routcount>0)
  {
    printf(" %d values offscale for histogram plot.\n",routcount);
  }
  
  printf("Histogram left + right raw count %d\n",icount);
  printf("  weighted count %d\n",rcount);
  
  sprintf(outfilename,"%s/Histograms.csv",outdirname);
  rfile=fopen(outfilename,"wt");

  fprintf(rfile,"\"hoffset\",\"hraw\",\"haes\"\n");
  
  ii=0;
  do
  {
    fprintf(rfile,"%6.4lf, ", (double)(ii-250)/hscale);
    fprintf(rfile,"%6.4lf, ", 100.0*(double)hist_raw[ii]/(double)icount);
    fprintf(rfile,"%6.4lf\n", 100.0*(double)hist_wet[ii]/(double)rcount);
    ii++;
  } while (ii<500);
  
  fclose(rfile);
  
  /* Now find 95% included range */
    
  ii=250; isum_raw=0; isum_wet=0; iw=0;
  ilimit=(int)(0.95*(double)icount);
  rlimit=(int)(0.95*(double)rcount);
  
  isum_raw=hist_raw[ii];
  do
  {
    iw++;
    isum_raw += hist_raw[ii-iw] + hist_raw[ii+iw];
    
    if ( isum_raw >= ilimit)
    {
      printf("Limit raw of %d at +/- %d [ %5.3lf percent]\n",isum_raw,iw,(double)iw/hscale);
      raw_2sig=(double)iw/hscale;
      iw=9999;
      
    }
  } while (iw<250);
      
  isum_wet=hist_wet[ii];
  iw=0;
  
  do
  {
    iw++;
    isum_wet += hist_wet[ii-iw] + hist_wet[ii+iw];
    
    if ( isum_wet >= rlimit)
    {
      printf("Limit weighted %d at +/- %d [ %5.3lf percent]\n",isum_wet,iw,(double)iw/hscale);
      aes_2sig=(double)iw/hscale;
      iw=9999;
    }
  } while (iw<250);
   
}

double get_weight(double fin)
{
  double answer, finp;
  
  /* This is a 6th order polynomial fit to the weighting specified by the AES */
  
  fin=log10(fin);
  
  answer = 0.61156 + 1.1433*fin;
  finp=fin*fin;
  answer += (-0.32082) * finp;
  finp=finp*fin;
  answer += (-1.1668) * finp;
  finp=finp*fin;
  answer += 0.24473 * finp;
  finp=finp*fin;
  answer += 0.40948 * finp;
  finp=finp*fin;
  answer += (-0.14516) * finp;
  
  return answer;
}

void regenerate(void)
{
  double dphi,fnow,phinow;
  double ai_left,aq_left,ai_right,aq_right;
  double spnow, cpnow, wnow;
  int ii,icf;
  
  icf=0;
  
  do
  {
    fnow=fsnow[icf];
    wnow=get_weight(fnow);
    ai_left =wnow*left_a[icf];  aq_left =wnow*left_q[icf];
    ai_right=wnow*right_a[icf]; aq_right=wnow*right_q[icf];
    dphi=0.01*2.0*3.1415927*fnow;
    phinow=0.0; 
    wnow=get_weight(fnow);
    ii=0;
    do
    {
      spnow=sin(phinow); cpnow=cos(phinow);
      rec_left[ii]  += ai_left*spnow  + aq_left*cpnow;
      rec_right[ii] += ai_right*spnow + aq_right*cpnow;
      phinow+=dphi;
      ii++;
    } while (ii<901);
    icf++;
  } while (fnow<49.8);
}

void do_transform(void)
{
  double fbase, tt, tstep, dphi, dphibase, fnow, phinow;
  double  ai_left, aq_left, ai_right, aq_right;
  double wscale, spnow,cpnow;
  double lamp, ramp, gw;
  int ii, ikeep;
  
  fbase=1.0/9.0;  tstep=0.01;  wscale=2.0/900.0;
  dphibase=2.0*3.1415927*fbase*tstep;
  fnow=fbase; dphi=dphibase;
  printf("Now doing spectrum of variations - please wait...\n");
  
  ikeep=0;
  
  sprintf(outfilename,"%s/Spectra.csv",outdirname);
  outfile=fopen(outfilename,"wt");

  fprintf(outfile,"\"freq\",\"sleft\",\"sright\",\"wsleft\",\"wsright\"\n");
  do
  {
    ai_left=0.0; aq_left=0.0;  ai_right=0.0; aq_right=0.0; tt=0.5; ii=0; phinow=0.0;
    do
    {
      spnow=sin(phinow); cpnow=cos(phinow);
      ai_left += left_rate[ii]*spnow; aq_left += left_rate[ii]*cpnow;
      ai_right += right_rate[ii]*spnow; aq_right += right_rate[ii]*cpnow;
      phinow+=dphi;
      ii++;
      tt += tstep;
    } while (ii<901);
  
    fprintf(outfile,"%6.4lf, ",fnow);
    
    lamp=wscale*sqrt(ai_left*ai_left+aq_left*aq_left);
    ramp=wscale*sqrt(ai_right*ai_right+aq_right*aq_right);
    fprintf(outfile,"%6.4lf, %6.4lf, ",lamp,ramp);
    gw=get_weight(fnow);
    lamp=gw*lamp; ramp=gw*ramp;
    fprintf(outfile,"%6.4lf, %6.4lf\n",lamp,ramp);
    fsnow[ikeep]=fnow; 
    left_a[ikeep] =wscale*ai_left;  left_q[ikeep] =wscale*aq_left;
    right_a[ikeep]=wscale*ai_right; right_q[ikeep]=wscale*aq_right;   
    ikeep++;
    fnow+=fbase;
    dphi+=dphibase;
    
  } while (fnow < 50.0); 
  fclose(outfile);
}

void highpass(void)
{
  int ii;
  
  /* Very simple 1st order highpass filter to reduce rumble */
  
  double fhp,tau,delta,lf_left,lf_right,dleft,dright;
  
  fhp=200.0; /* filter turnover */  
  tau=1.0/(2.0*3.1415927*fhp); /* filter time constant */
  delta=1.0-exp(-dt/tau);
  printf("delta = %6.4lf\n",delta);
  lf_left=0.0; lf_right=0.0;
  
  ii=1;
  do
  {
    dleft=left[ii] - lf_left;
    lf_left+=delta*dleft;
    left[ii]-=lf_left;
    dright=right[ii] - lf_right;
    lf_right+=delta*dright;
    right[ii]-=lf_right;
    ii++;
  } while (ii<samples_per_block);
}



double get_fi(double tfistart, double tfirlong)
{
  int iistart,iistop;
  int last_left,last_right, first_left,first_right;
  int ii,count_left,count_right;
  int ipush;
  double ffound,f0_left,f0_right;
  double start_nibble, end_nibble;
  double pc_left,pc_right;
  
  last_left=-10;  last_right=-10;
  first_left=-10; first_right=-10;
  count_left=0;   count_right=0;
  ipush=1;  /* This pushes past the risk of a double count due to noise */
  
  iistart= (int) ( tfistart/dt );
  iistop = iistart + (int)( tfirlong/dt );
  
  ii=iistart;
  do
  {
    
    if( left[ii-1]<=0.0 )
    {
      if ( left[ii] > 0.0)
      {
        last_left=ii;
        if(first_left<0)
        {
          first_left=ii;
        }
        else
        {
          count_left++;
          last_left=ii;
        }
        ii+=ipush; 
      }
    }
    
    ii++;
  } while (ii<iistop);
  
  
  ii=iistart;
  do
  {
    
    if( right[ii-1]<=0.0 )
    {
      if ( right[ii] > 0.0)
      {
        last_right=ii;
        if(first_right<0)
        {
          first_right=ii;
        }
        else
        {
          count_right++;
          last_right=ii;
        }
        
        ii+=ipush; 
      }
    }
    
    ii++;
    
  } while (ii<iistop);   
  
    ffound=(double)(last_left-first_left);
    if(count_left>0)
    {
      ffound=ffound/(double)(count_left);
      f0_left=(double)pairs_per_sec/ffound;
    }
    else
    {
      f0_left=0.0;
    }
    
    ffound=(double)(last_right-first_right);
    
    if(count_right>0)
    {
      
      ffound=ffound/(double)(count_right);
      f0_right=(double)pairs_per_sec/ffound;
    }
    else
    {
      f0_right=0.0;
    }
    
    start_nibble=interpolate(left[first_left-1],left[first_left]);
    end_nibble=interpolate(left[last_left-1],left[last_left]);
    
    ffound=(double)(last_left-first_left)-start_nibble+end_nibble;
    
    if(count_left>0)
    {
      ffound=ffound/(double)(count_left);
      f0_left=(double)pairs_per_sec/ffound;
    }
    else
    {
      f0_left=0.0;
    }

    start_nibble=interpolate(right[first_right-1],right[first_right]);
    end_nibble=interpolate(right[last_right-1],right[last_right]);
  
    ffound=(double)(last_right-first_right)-start_nibble+end_nibble;
    
    if(count_right>0)
    {
      ffound=ffound/(double)(count_right);
      f0_right=(double)pairs_per_sec/ffound;
    }
    else
    {
      f0_right=0.0;
    }
    
    if(rms_left>rms_right)
    {
      ffound=f0_left;
    }
    else
    {
      ffound=f0_right;
    }
    
    pc_left =f0_left/f0_counter;  pc_left=100.0*(pc_left-1.0);
    pc_right=f0_right/f0_counter; pc_right=100.0*(pc_right-1.0);
    
    left_rate[i]=pc_left;  right_rate[i]=pc_right;
    
  return ffound;
}


double interpolate(double ia,double ib)
{
  double answer;
  answer= (-ia)/(ib-ia);
  return answer;
}



double find_counter_frequency(void)
{
  /* This proceedure employs a fringe counting method to
     determine the frequency of the periodic signal.
     It then sets f0 to be the found value for whichever
     channel has the highest power level.
  */
  int last_left,last_right, first_left,first_right;
  int ii,count_left,count_right;
  int ipush;
  double ffound,f0_left,f0_right;
  last_left=-10;  last_right=-10;
  first_left=-10; first_right=-10;
  count_left=0;   count_right=0;
  ipush=1;  /* This pushes past the risk of a double count due to noise */
  
  ii=1;
  do
  {
    if( left[ii-1]<=0.0 )
    {
      if ( left[ii] > 0.0)
      {
        last_left=ii;
        
        if(first_left<0)
        {
          first_left=ii;
        }
        else
        {
          count_left++;
          last_left=ii;
        }
        
        ii+=ipush; 
      }
    }
    
    ii++;
  } while (ii<samples_per_block);
  
  ii=1;
  do
  {
    if( right[ii-1]<=0.0 )
    {
      if ( right[ii] > 0.0)
      {
        last_right=ii;
        
        if(first_right<0)
        {
          first_right=ii;
        }
        else
        {
          count_right++;
          last_right=ii;
        }
        
        ii+=ipush; 
      }
    }
    
    ii++;
  } while (ii<samples_per_block);   
   
    printf("Left  %d to %d has %d cycles\n",first_left,last_left,count_left);
    printf("Right %d to %d has %d cycles\n",first_right,last_right,count_right);
    
    ffound=(double)(last_left-first_left);
    
    if(count_left>0)
    {
      ffound=ffound/(double)(count_left);
      f0_left=(double)pairs_per_sec/ffound;
    }
    else
    {
      f0_left=0.0;
    }
    
    ffound=(double)(last_right-first_right);
    
    if(count_right>0)
    {
      
      ffound=ffound/(double)(count_right);
      f0_right=(double)pairs_per_sec/ffound;
    }
    else
    {
      f0_right=0.0;
    }
    
    printf("Frequencies %8.4lf Hz Left  %8.4lf Right\n",f0_left,f0_right);  
    
    if(rms_left>rms_right)
    {
      ffound=f0_left;
      printf("Left chosen\n");
    }
    else
    {
      ffound=f0_right;
      printf("Right chosen\n");
    }
    
  return ffound;
}



void load_data_block16(void)
{
  
  /* This procedure finds and loads the selected block of sample
     data and then converts the raw data into a set of real sample
     values for analysis. */
  
  int block_size_read;
  int icut,icut2;
  int io,ii,lt,rt;
  double samscale, bscale;
  
  bscale=1.0/(double)samples_per_block;
  
  icut=32768; icut2=icut*2;
  samscale=1.0/(double)icut;
  dc_left=0.0; dc_right=0.0;
  rms_left=0.0; rms_right=0.0;
  block_size_read=fread(datablock,sizeof(char),block_size,infile);
  
  printf("Block size read = %d\n",block_size_read);
  
  ii=0; io=0;
  
  do
  {
    lt=  256*(int)datablock[ii+1] + (int)datablock[ii];
    ii+=2;
    
    rt= 256*(int)datablock[ii+1] + (int)datablock[ii];
    ii+=2;
    
    if (lt>=icut) lt-=icut2;
    if (rt>=icut) rt-=icut2;
    left[io]=samscale*(double)lt;
    right[io]=samscale*(double)rt;
    rms_right+=right[io]*right[io];
    rms_left+=left[io]*left[io];
    
    io++;
    
  } while (io<samples_per_block);
  
  dc_right=dc_right*bscale;
  dc_left=dc_left*bscale;
  rms_left=rms_left*bscale;
  rms_right=rms_right*bscale;
  
  
  rms_left=sqrt(rms_left);
  rms_right=sqrt(rms_right);
  
  
}


void load_data_block24(void)
{
  
  /* This procedure finds and loads the selected block of sample
     data and then converts the raw data into a set of real sample
     values for analysis. */
  
  int block_size_read;
  int icut,icut2;
  int io,ii,lt,rt;
  double samscale, bscale;
  
  bscale=1.0/(double)samples_per_block;
  
  icut=32768*256; icut2=icut*2;
  samscale=1.0/(double)icut;
  dc_left=0.0; dc_right=0.0;
  rms_left=0.0; rms_right=0.0;
  block_size_read=fread(datablock,sizeof(char),block_size,infile);
  
  printf("Block size read = %d\n",block_size_read);
  
  ii=0; io=0;
  
  do
  {
    lt=65536*(int)datablock[ii+2] + 256*(int)datablock[ii+1] + (int)datablock[ii];
    ii+=3;
    
    rt=65536*(int)datablock[ii+2] + 256*(int)datablock[ii+1] + (int)datablock[ii];
    ii+=3;
    
    if (lt>=icut) lt-=icut2;
    if (rt>=icut) rt-=icut2;
    left[io]=samscale*(double)lt;
    right[io]=samscale*(double)rt;
    rms_right+=right[io]*right[io];
    rms_left+=left[io]*left[io];
    
    io++;
    
  } while (io<samples_per_block);
  
  dc_right=dc_right*bscale;
  dc_left=dc_left*bscale;
  rms_left=rms_left*bscale;
  rms_right=rms_right*bscale;
  
  rms_left=sqrt(rms_left);
  rms_right=sqrt(rms_right);
}

double read_header(void)
{
  /* This procedure reads the header if it is a WAV file */
  double result;
  
  infile=fopen(infilename,"rd");
  
  get4(ChunkID);
  printf("ChunkID = %s\n",ChunkID);
  ChunkSize=get4int();
  printf("Chunk Size = %d\n",ChunkSize);

  get4(Format);
  printf("Format = %s\n\n",Format);
  get4(Subchunk1ID);
  printf("Subchunk1ID = %s\n",Subchunk1ID);
  Subchunk1Size=get4int();
  printf("Subchunk1Size = %d [should be 16 for LPCM]\n",Subchunk1Size);
    
  AudioFormat=get2int();
  printf("AudioFormat = %d [should be 1 for LPCM]\n",AudioFormat);
    
  NumChannels=get2int();
  printf("Number of Channels = %d\n",NumChannels);
  SampleRate=get4int();
  printf("Sample Rate = %d Samples per sec\n",SampleRate);
  ByteRate=get4int();
  printf("Byte Rate = %d Bytes per sec\n",ByteRate);
    
  BlockAlign=get2int();
  printf("Block Align = %d\n\n",BlockAlign);
  BitsPerSample=get2int();
  printf("Bits per sample = %d\n",BitsPerSample);
  BytesPerSample=BitsPerSample/8;
  printf("Bytes per sample = %d\n\n",BytesPerSample);
    
  get4(Subchunk2ID);
  printf("Subchunk2ID = %s\n",Subchunk2ID);
  Subchunk2Size=get4int();
  printf("Subchunk2Size = %d [Number of Data payload bytes]\n\n",Subchunk2Size);
  
  result=0;
  
  if(Subchunk2ID[0]=='d') result=1;
  if(Subchunk2ID[1]=='a') result+=1;
  if(Subchunk2ID[2]=='t') result+=1;
  if(Subchunk2ID[3]=='a') result+=1;
  
  if(result==4)
  {
    printf("WAV data start located OK\n");
    
    result=(double)Subchunk2Size/(double)ByteRate;
    pairs_per_sec=SampleRate;
    
    if(NumChannels==1)
    {
      printf("MONO Samples per sec = %d\n",pairs_per_sec);
    }
    else
    {
      printf("Sample pairs per sec = %d\n",pairs_per_sec);
    }
    
    
    printf("Duration = %6.4lf sec\n\n",result);
    
  }
  else
  {
    printf("Not identified as WAV data!\n");
    result=0.0;
  }
  
 fclose(infile);
    
  return result;
}

int get_start_time(void)
{
  /* This procedure asks the user what time (in seconds from the
     start of the file) is the beginning of the chunk to be loaded
     and analysed. */
  
  int ir, irtemp;
  ir=0;
  printf("Enter start time of section to transform [seconds]\n   => ");
  scanf("%lf",&tstart);
  printf("\nRequested start time %7.2lf\n",tstart);
  
  if (tstart>(total_time-10.05))
  {
    ir=-1;
    printf("\nTime requested goes beyond end of file!\n");
  }
  
  if (tstart<0.0)
  {
    ir=-2;
    printf("\nTime requested before file starts!\n");
  }
  
  if (ir==0)
  {
    irtemp=(int)(tstart*(double)SampleRate);
    ir=BlockAlign*(int)irtemp;
    printf("Start sample pair %d = At Byte %d\n",irtemp,ir);
    
  }
  return ir;
}


int get2int(void)
{
  int result;
  unsigned char cip1,cip2;
  
  fscanf(infile,"%c",&cip1);
  fscanf(infile,"%c",&cip2);

  result=(int)cip2;
  result=(int)cip1+256*result;

  return result;
} 

int get4int(void)
{
  int result;
  unsigned char cip1,cip2,cip3,cip4;
  
  fscanf(infile,"%c",&cip1);
  fscanf(infile,"%c",&cip2);
  fscanf(infile,"%c",&cip3);
  fscanf(infile,"%c",&cip4);
    
  result=(int)cip4;
  result=(int)cip3+256*result;
  result=(int)cip2+256*result;
  result=(int)cip1+256*result;
  
  return result;
} 
void get4(char* outstring)
{
  int isp;
  char csp;
  isp=0;
  do
  {
    fscanf(infile,"%c",&csp);
    outstring[isp]=csp;
    isp++;
  } while (isp<4);
  outstring[4]=(char)0;

}


